package com.vendor.burn.packagers.nand.ip;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import com.vendor.burn.Messages;
import com.vendor.burn.model.FileSystem;
import com.vendor.burn.model.PartitionItem;
import com.vendor.burn.model.config.PreferenceConfig;
import com.vendor.burn.model.define.IBurner;
import com.vendor.burn.model.define.IFastbootProgressMonitor;
import com.vendor.burn.model.devices.unknown.UDProperties;
import com.vendor.burn.model.devices.unknown.UnknownDevice;
import com.vendor.burn.packagers.nand.BlockType;
import com.vendor.burn.packagers.nand.FlashOption;
import com.vendor.burn.packagers.nand.NandCRandom;
import com.vendor.burn.packagers.nand.PageType;
import com.vendor.burn.packagers.nand.ReverseBlockSizeSector;
import com.vendor.burn.packagers.nand.ReversePageEccSector;
import com.vendor.burn.packagers.nand.ecc.ECC600;
import com.vendor.burn.packagers.nand.ecc.ECCBase;
import com.vendor.burn.packagers.nand.ecc.ECCType;
import com.vendor.burn.packagers.nand.ip.NandIP620;
import com.vendor.burn.packagers.nand.ip.NandIPCommon;
import com.vendor.burn.preferences.FastbootPreferences;
import com.vendor.burn.utilitily.BytesUtils;
import com.vendor.burn.utilitily.FileUtils;
import com.vendor.burn.utilitily.ToolUtils;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.List;
import org.apache.commons.io.FileUtils;
import org.eclipse.core.runtime.IProgressMonitor;

public class NandIP600 extends NandIPCommon {
  private IBurner burner;
  
  private String boardType;
  
  private static int NANDIP_600 = 1536;
  
  static final ReversePageEccSector[] fmc_page_ecc_reverse_sector = new ReversePageEccSector[] { new ReversePageEccSector(NANDIP_600, PageType.Size2K, ECCType.Ecc4bit, 0), new ReversePageEccSector(NANDIP_600, PageType.Size2K, ECCType.Ecc24bit1k, 1), new ReversePageEccSector(NANDIP_600, PageType.Size4K, ECCType.Ecc4bit, 2), new ReversePageEccSector(NANDIP_600, PageType.Size4K, ECCType.Ecc24bit1k, 3), new ReversePageEccSector(NANDIP_600, PageType.Size8K, ECCType.Ecc24bit1k, 4), new ReversePageEccSector(NANDIP_600, PageType.Size8K, ECCType.Ecc40bit1k, 5), new ReversePageEccSector(NANDIP_600, PageType.Size16K, ECCType.Ecc40bit1k, 6), new ReversePageEccSector(NANDIP_600, PageType.Size16K, ECCType.Ecc64bit1k, 7), new ReversePageEccSector(NANDIP_600, PageType.Size8K, ECCType.Ecc64bit1k, 8) };
  
  private static final ReverseBlockSizeSector[] fmc_blocksize_reverse_sector = new ReverseBlockSizeSector[] { new ReverseBlockSizeSector(NANDIP_600, BlockType.BlockSize64, 0), new ReverseBlockSizeSector(NANDIP_600, BlockType.BlockSize128, 1), new ReverseBlockSizeSector(NANDIP_600, BlockType.BlockSize256, 2), new ReverseBlockSizeSector(NANDIP_600, BlockType.BlockSize512, 3) };
  
  public FlashOption[] getAvailableOptions(IBurner burner) {
    this.burner = burner;
    UDProperties properties = ((UnknownDevice)burner.getBurnContext().getDeviceSetting()).getProperties();
    this.boardType = properties.queryForString("TYPE");
    Integer spiNand = properties.queryForInteger("SPINANDBURN");
    if (this.boardType != null && this.boardType.equalsIgnoreCase("CAM")) {
      if (spiNand != null && spiNand.intValue() != -1)
        return new FlashOption[] { new FlashOption(ECCType.Ecc4bit, PageType.Size2K, 60), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, 116), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, 88), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, 200) }; 
      return new FlashOption[] { new FlashOption(ECCType.Ecc4bit, PageType.Size2K, 64), new FlashOption(ECCType.Ecc16bit1k, PageType.Size2K, 88), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, 128), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, 128), new FlashOption(ECCType.Ecc16bit1k, PageType.Size4K, 144), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, 200), new FlashOption(ECCType.Ecc24bit1k, PageType.Size8K, 368) };
    } 
    return new FlashOption[] { new FlashOption(ECCType.Ecc4bit, PageType.Size2K, 64), new FlashOption(ECCType.Ecc16bit1k, PageType.Size2K, 88), new FlashOption(ECCType.Ecc24bit1k, PageType.Size2K, 128), new FlashOption(ECCType.Ecc4bit, PageType.Size4K, 128), new FlashOption(ECCType.Ecc16bit1k, PageType.Size4K, 144), new FlashOption(ECCType.Ecc24bit1k, PageType.Size4K, 200), new FlashOption(ECCType.Ecc24bit1k, PageType.Size8K, 368), new FlashOption(ECCType.Ecc40bit1k, PageType.Size8K, 600), new FlashOption(ECCType.Ecc40bit1k, PageType.Size16K, 1200) };
  }
  
  public byte[] genPageData(byte[] pagebuf, FlashOption option, int oobSize) {
    ECC600 ecc = new ECC600();
    if (option.getEccType() == ECCType.Ecc4bit) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 14); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 14); 
    } 
    if (option.getEccType() == ECCType.Ecc16bit1k) {
      if (option.getPageType() == PageType.Size2K) {
        if (oobSize == 8)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 6, 28); 
        if (oobSize == 32)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 28); 
        if (oobSize == 88)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 28); 
      } else if (option.getPageType() == PageType.Size4K) {
        if (oobSize == 16)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 14, 28); 
        if (oobSize == 32)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 28); 
        if (oobSize == 128)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 28); 
        if (oobSize == 144)
          return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 28); 
      } 
    } else if (option.getEccType() == ECCType.Ecc24bit1k) {
      if (option.getPageType() == PageType.Size2K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size4K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 42); 
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 42); 
    } else if (option.getEccType() == ECCType.Ecc40bit1k) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 70); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 70); 
    } else if (option.getEccType() == ECCType.Ecc64bit1k) {
      if (option.getPageType() == PageType.Size8K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 112); 
      if (option.getPageType() == PageType.Size16K)
        return genPageDataBytes((ECCBase)ecc, option, pagebuf, 30, 112); 
    } 
    return null;
  }
  
  public void make(String outputPath, FlashOption option, List<PartitionItem> partitions, int blockSize, boolean isSelectRandom, IFastbootProgressMonitor fastbootMonitor, IProgressMonitor monitor) {
    fastbootMonitor.infoln("\nStart to make nand burner image...");
    fastbootMonitor.infoln("Options: " + option.toString() + "\n");
    monitor.beginTask(Messages.NandImageDialog_Title, partitions.size());
    int randomPageNum = (isSelectRandom && blockSize > -1) ? (randomPageNum = blockSize / 8192) : -1;
    label161: for (PartitionItem item : partitions) {
      if (monitor.isCanceled())
        return; 
      boolean itemState = (!item.isChecked() || item.getFile() == null);
      if (itemState) {
        monitor.worked(1);
        continue;
      } 
      differenceBlockSize(item, blockSize);
      fastbootMonitor.info("Make image for '" + item.getFile().getName() + "'...");
      File file = (item.getNewTempFile() == null) ? item.getFile() : item.getNewTempFile();
      int pageCheckSize = (item.getFileSystem() == FileSystem.yaffs) ? (pageCheckSize = option.getPageType().getPageSize() + option.getOOBSize()) : option.getPageType().getPageSize();
      int pageCount = (item.getFileSystem() == FileSystem.yaffs) ? (int)(file.length() / pageCheckSize) : (int)((file.length() + pageCheckSize - 1L) / pageCheckSize);
      File outFile = FileUtils.getFile(new String[] { outputPath + outputPath + File.separator + ".burn" });
      if (outFile.exists())
        FileUtils.deleteFile(outFile); 
      RandomAccessFile inRanFile = null;
      RandomAccessFile outRanFile = null;
      boolean ret = true;
      try {
        FileUtils.checkFilePath(file);
        inRanFile = new RandomAccessFile(file, "r");
        outRanFile = new RandomAccessFile(outFile, "rw");
        int pageMakeSize = option.getPageType().getPageSize() + option.getOOBSize();
        long start = item.getStart();
        int reversePageEccEec = 0;
        int reverseBlockEec = 0;
        boolean page0ReverseFlag = false;
        boolean page1ReverseFlag = false;
        boolean nandPinSaveMpde = false;
        if (ToolUtils.isCmdLine()) {
          nandPinSaveMpde = PreferenceConfig.isImgOpenPinSaveMode();
        } else {
          nandPinSaveMpde = FastbootPreferences.isNandPinSaveMpde();
        } 
        boolean supportPinSaveMode = false;
        boolean supportEmpty = false;
        String curPinSaveMode = null;
        if (this.boardType.equalsIgnoreCase("CAM")) {
          boolean isNand = (this.curSelectedNandWay != null && this.curSelectedNandWay.equalsIgnoreCase("Nand"));
          if (isNand) {
            curPinSaveMode = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getNandPinSaveMode();
            boolean pinSaveMode = (curPinSaveMode != null && !curPinSaveMode.isEmpty());
            if (pinSaveMode)
              supportPinSaveMode = true; 
            supportEmpty = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportNandEmpty();
          } else if (this.curSelectedNandWay.equalsIgnoreCase("SPINand")) {
            curPinSaveMode = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).getSpiNandPinSaveMode();
            boolean pinSaveMode = (curPinSaveMode != null && !curPinSaveMode.isEmpty());
            if (pinSaveMode)
              supportPinSaveMode = true; 
            supportEmpty = ((UnknownDevice)this.burner.getBurnContext().getDeviceSetting()).isSupportSPINandEmpty();
          } 
        } 
        boolean isSupport = (nandPinSaveMpde && supportPinSaveMode && start == 0L);
        if (isSupport) {
          boolean isV2 = (curPinSaveMode != null && curPinSaveMode.equalsIgnoreCase(getPinSaveModeArray()[0]));
          ReversePageEccSector[] sector = isV2 ? NandIP620.getNfc620PageEccReverseSector() : (ReversePageEccSector[])fmc_page_ecc_reverse_sector.clone();
          PageType pageType = option.getPageType();
          ECCType eccType = option.getEccType();
          ReversePageEccSector pageSizeEccReverseSector = getPageSizeEccReverseSectorByInputSector(pageType, eccType, sector);
          if (pageSizeEccReverseSector != null)
            reversePageEccEec = pageSizeEccReverseSector.getReverse_sec(); 
          ReverseBlockSizeSector blockSizeReverseSector = getBlockSizeReverseSector(blockSize, pageType);
          if (blockSizeReverseSector != null)
            reverseBlockEec = blockSizeReverseSector.getReverse_sec(); 
        } 
        for (int i = 0; i < pageCount; i++) {
          if (monitor.isCanceled()) {
            closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
            FileUtils.deleteFile(item.getNewTempFile());
            if (!ret) {
              FileUtils.deleteFile(outFile);
              monitor.worked(1);
              continue label161;
            } 
            return;
          } 
          byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
          try {
            inRanFile.seek((i * pageCheckSize));
            doEmptyForCAM(option, item, pageCheckSize, inRanFile, supportEmpty, pagebuf);
          } catch (IOException e) {
            fastbootMonitor.warnln("FAILED! (Read data error.)");
            ret = false;
            break;
          } 
          byte[] data = null;
          try {
            data = genPageData(pagebuf, option, option.getOOBSize());
            if (randomPageNum > -1) {
              NandCRandom random = new NandCRandom();
              random.page_random_gen(data, option.getPageType().getPageSize(), option.getOOBSize(), i % randomPageNum);
            } 
            if (isSupport) {
              if (!page0ReverseFlag) {
                int page0ReverseOff = 32 * reversePageEccEec;
                reverseBuf(data, page0ReverseOff, 32);
              } 
              boolean reverse = (page0ReverseFlag && !page1ReverseFlag);
              if (reverse) {
                int page1ReverseOff = 32 * reverseBlockEec;
                reverseBuf(data, page1ReverseOff, 32);
                page1ReverseFlag = true;
              } 
              page0ReverseFlag = true;
            } 
            outRanFile.seek((i * pageMakeSize));
            outRanFile.write(data);
          } catch (Exception e) {
            fastbootMonitor.warnln("FAILED! (Data convert error.)");
            ret = false;
            break;
          } 
        } 
        ret = setEmptyFlag(option, blockSize, fastbootMonitor, item, pageCheckSize, pageCount, outRanFile, pageMakeSize, supportEmpty);
        if (!ret) {
          closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
          FileUtils.deleteFile(item.getNewTempFile());
          if (!ret) {
            FileUtils.deleteFile(outFile);
            monitor.worked(1);
            continue;
          } 
          return;
        } 
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } catch (FileNotFoundException e) {
        fastbootMonitor.warnln("FAILED! (Source file not found.)");
        monitor.worked(1);
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
        } 
        continue;
      } catch (IOException iOException) {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } finally {
        closeTransportStream(fastbootMonitor, inRanFile, outRanFile);
        FileUtils.deleteFile(item.getNewTempFile());
        if (!ret) {
          FileUtils.deleteFile(outFile);
          monitor.worked(1);
          continue;
        } 
      } 
      fastbootMonitor.info("OK!");
      fastbootMonitor.warnln("(Start Address = 0x" + 
          Long.toHexString(item.getStart()) + ", Length = 0x" + 
          Long.toHexString(item.getLength()) + ")");
      monitor.worked(1);
    } 
    fastbootMonitor
      .infoln("\nImage creating operation ends. All of the selected partitions are processed.");
    monitor.done();
  }
  
  private void doEmptyForCAM(FlashOption option, PartitionItem item, int pageCheckSize, RandomAccessFile inRanFile, boolean supportEmpty, byte[] pagebuf) throws IOException {
    int read = inRanFile.read(pagebuf, 0, pageCheckSize);
    if (read < 0)
      System.out.println("Read Error."); 
    boolean isEmptyAndCAM = (this.boardType != null && this.boardType.equalsIgnoreCase("CAM") && supportEmpty);
    if (isEmptyAndCAM) {
      int camPageEmptyFlag = option.getPageType().getPageSize() + 32;
      if (item.getFileSystem() == FileSystem.ubiFs) {
        if (checkBufferFF(pagebuf, pageCheckSize)) {
          pagebuf[camPageEmptyFlag - 1] = 0;
          pagebuf[camPageEmptyFlag - 2] = 0;
        } 
      } else {
        pagebuf[camPageEmptyFlag - 1] = 0;
        pagebuf[camPageEmptyFlag - 2] = 0;
      } 
    } 
  }
  
  private boolean setEmptyFlag(FlashOption option, int blockSize, IFastbootProgressMonitor fastbootMonitor, PartitionItem item, int pageCheckSize, int pageCount, RandomAccessFile outRanFile, int pageMakeSize, boolean supportEmpty) {
    boolean ret = true;
    boolean isSupportEmpty = (this.boardType != null && this.boardType.equalsIgnoreCase("CAM") && supportEmpty);
    if (isSupportEmpty) {
      int pageNumInclData = pageCount;
      int pagePerBlock = blockSize / option.getPageType().getPageSize();
      int pageNumTotalBlock = pageNumInclData & (pagePerBlock - 1 ^ 0xFFFFFFFF);
      pageNumTotalBlock += ((pageNumInclData & pagePerBlock - 1) > 0) ? pagePerBlock : 
        0;
      int pageNumFillNull = 0;
      if (pageNumTotalBlock - pageNumInclData > 0)
        pageNumFillNull = pageNumTotalBlock - pageNumInclData; 
      while (pageNumFillNull-- > 0) {
        byte[] pagebuf = BytesUtils.genBytesWithFF(pageMakeSize);
        int camPageEmptyFlag = option.getPageType().getPageSize() + 32;
        if (item.getFileSystem() == FileSystem.ubiFs) {
          if (checkBufferFF(pagebuf, pageCheckSize)) {
            pagebuf[camPageEmptyFlag - 1] = 0;
            pagebuf[camPageEmptyFlag - 2] = 0;
          } 
        } else {
          pagebuf[camPageEmptyFlag - 1] = 0;
          pagebuf[camPageEmptyFlag - 2] = 0;
        } 
        byte[] data = null;
        try {
          data = genPageData(pagebuf, option, option.getOOBSize());
          if (data != null)
            outRanFile.write(data); 
        } catch (Exception e) {
          fastbootMonitor.warnln("FAILED! (Write Empty flag error.)");
          ret = false;
          break;
        } 
      } 
    } 
    return ret;
  }
  
  private ReversePageEccSector getPageSizeEccReverseSectorByInputSector(PageType pagetype, ECCType ecctype, ReversePageEccSector[] sector) {
    for (int i = 0; i < sector.length; i++) {
      if (sector[i].getEcctype() == ecctype && sector[i]
        .getPagetype() == pagetype)
        return sector[i]; 
    } 
    return null;
  }
  
  private ReverseBlockSizeSector getBlockSizeReverseSector(int blocksize, PageType pageType) {
    int pageNum = blocksize / pageType.getPageSize();
    ReverseBlockSizeSector[] sector = fmc_blocksize_reverse_sector;
    for (int i = 0; i < sector.length; i++) {
      if (sector[i].getBlocktype().getSize() == pageNum)
        return sector[i]; 
    } 
    return null;
  }
  
  private static void reverseBuf(byte[] buf, int startOff, int reverseLen) {
    for (int i = startOff; i < startOff + reverseLen; i++)
      buf[i] = (byte)(buf[i] ^ 0xFFFFFFFF); 
  }
  
  public boolean checkBufferFF(byte[] pagebuf, int length) {
    for (int index = 0; index < length; index++) {
      if (pagebuf[index] != -1)
        return true; 
    } 
    return false;
  }
  
  public String toString() {
    return "Nand IP 600";
  }
}
